/*
 * readpass.c: this file is part of the tlock program.
 *
 * tlock is a program to lock the terminal with user's login password.
 * Copyright (C) 2005 2006 2007 2008 2009 2010 Prasad J Pandit.
 *
 * tlock is a free software; you can redistribute it and/or modify it under
 * the terms of GNU General Public Licence as published by Free Software
 * Foundation; either version 2 of the licence, or (at your option) any later
 * version.
 *
 * tlock is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public Licence for more
 * details.
 *
 * You should have received a copy of the GNU General Public Licence along
 * with tlock; if not, write to Free Software Foundation, Inc., 51 Franklin
 * St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <term.h>

#include <readpass.h>

/* start and stop echoing of input characters on the console */
static int strt_echo (void);
static int stop_echo (void);

unsigned char cerase;   /* erase character of terminal */

/*
 * readpass: this function reads in the password from standard input, and
 * returns a pointer to password, or NULL in case of an error
 */
char *
readpass (const char *prompt, char display, size_t len)
{
    unsigned int i = 0;
    char *pass = NULL, c = ' ';

    if (stop_echo () < 0)
        return NULL;
    if (!(pass = calloc (len, sizeof (char))))
        return NULL;

    printf ("%s", prompt);
    while (((c = getchar ()) != '\n') && (i < len))
    {
        if ((c != cerase) && (c != '\b'))
        {
             pass[i++] = c;
             putchar (display);
        }
        else if (i > 0)
        {
            putchar ('\b');
            putchar (' ');
            putchar ('\b');
            i--;
        }
    }
    pass[i] = '\0';
    if (strt_echo () < 0)
    {
        free (pass);
        return NULL;
    }

    return pass;
}

/* stop_echo: turns off echo of input character on standard output */
static int
stop_echo (void)
{
    struct termios term;

    if (tcgetattr (STDOUT_FILENO, &term) < 0)
        return -1;

    cerase = term.c_cc[VERASE];
    term.c_iflag &= ~IXON;
    term.c_iflag |= IXOFF;
    term.c_lflag &= (~ECHO & ~ICANON & ~ISIG);
    if (tcsetattr (STDOUT_FILENO, TCSANOW, &term) < 0)
        return -1;

    return 0;
}

/* strt_echo: start echo of input character on standard output */
static int
strt_echo (void)
{
    struct termios term;

    if (tcgetattr (STDOUT_FILENO, &term) < 0)
        return -1;

    term.c_iflag |= IXON;
    term.c_iflag &= ~IXOFF;
    term.c_lflag |= ECHO | ICANON | ISIG;
    if (tcsetattr (STDOUT_FILENO, TCSANOW, &term) < 0)
        return -1;
    putchar ('\n');

    return 0;
}
